// ArrayPtr.h

#ifndef ArrayPtr_h
#define ArrayPtr_h

#include <functional>
#ifndef ArrayRef_h
#include "ArrayRef.h"
#endif

template < class Element > class ArrayPtr;

template < class Element >
class ArrayPtr< const Element >
  {
	public:
		typedef ::ArrayRef< const Element > 		ArrayRef;

		typedef ArrayRef::value_type					value_type;
		typedef ArrayRef::pointer						pointer;
		typedef ArrayRef::const_pointer				const_pointer;
		typedef ArrayRef::reference					reference;
		typedef ArrayRef::const_reference			const_reference;
		typedef ArrayRef::difference_type			difference_type;
		typedef ArrayRef::size_type					size_type;
		typedef ArrayRef::const_iterator				const_iterator;
		typedef ArrayRef::iterator						iterator;
		typedef ArrayRef::reverse_iterator			reverse_iterator;
		typedef ArrayRef::const_reverse_iterator	const_reverse_iterator;

	private:
		ArrayRef ref;
	
	public:
		ArrayPtr()																	{}
		ArrayPtr( pointer begin, pointer end )								: ref( begin, end )		{}
		// Plus the implicitly-generated copy constructor
		
		ArrayPtr& operator=( const ArrayPtr& s )							{ ref.Reseat( s.ref ); return *this; }
		
		bool Null() const															{ return ref.Null(); }
		
		const ArrayRef& operator*() const									{ Assert( !Null() ); return ref; }
		const ArrayRef *operator->() const									{ Assert( !Null() ); return ref.Address(); }

		reference operator[]( size_type n ) const							{ Assert( !Null() ); return ref[n]; }

		bool operator==( const ArrayPtr& r ) const						{ return ref.b == r.ref.b && ref.e == r.ref.e; }
		bool operator< ( const ArrayPtr& r ) const						{ return ref.b != r.ref.b
																									? std::less<pointer>()( ref.b, r.ref.b )
																						         : std::less<pointer>()( ref.e, r.ref.e ); }
		
		// Boilerplate comparisons:
			bool operator> ( const ArrayPtr& r ) const					{ return r < *this; }
			bool operator!=( const ArrayPtr& r ) const					{ return !( *this == r ); }
			bool operator<=( const ArrayPtr& r ) const					{ return !( *this > r ); }
			bool operator>=( const ArrayPtr& r ) const					{ return !( *this < r ); }
  };

template < class Element >
class ArrayPtr
  {
	public:
		typedef ::ArrayRef< Element > 				ArrayRef;

		typedef ArrayRef::value_type					value_type;
		typedef ArrayRef::pointer						pointer;
		typedef ArrayRef::const_pointer				const_pointer;
		typedef ArrayRef::reference					reference;
		typedef ArrayRef::const_reference			const_reference;
		typedef ArrayRef::difference_type			difference_type;
		typedef ArrayRef::size_type					size_type;
		typedef ArrayRef::const_iterator				const_iterator;
		typedef ArrayRef::iterator						iterator;
		typedef ArrayRef::reverse_iterator			reverse_iterator;
		typedef ArrayRef::const_reverse_iterator	const_reverse_iterator;

	private:
		ArrayRef ref;
	
	public:
		ArrayPtr()																	{}
		ArrayPtr( pointer begin, pointer end )								: ref( begin, end )		{}
		// Plus the implicitly-generated copy constructor
		
		ArrayPtr& operator=( const ArrayPtr& s )							{ ref.Reseat( s.ref ); return *this; }
		
		bool Null() const															{ return ref.Null(); }
		
		operator ArrayPtr< const value_type >()							{ return ArrayPtr< const value_type >( ref.b, ref.e ); }
		
		const ArrayRef& operator*() const									{ Assert( !Null() ); return ref; }
		const ArrayRef *operator->() const									{ Assert( !Null() ); return ref.Address(); }

		reference operator[]( size_type n ) const							{ Assert( !Null() ); return ref[n]; }

		bool operator==( const ArrayPtr& r ) const						{ return ref.b == r.ref.b && ref.e == r.ref.e; }
		bool operator< ( const ArrayPtr& r ) const						{ return ref.b != r.ref.b
																									? std::less<pointer>()( ref.b, r.ref.b )
																						         : std::less<pointer>()( ref.e, r.ref.e ); }
		
		// Boilerplate comparisons:
			bool operator> ( const ArrayPtr& r ) const					{ return r < *this; }
			bool operator!=( const ArrayPtr& r ) const					{ return !( *this == r ); }
			bool operator<=( const ArrayPtr& r ) const					{ return !( *this > r ); }
			bool operator>=( const ArrayPtr& r ) const					{ return !( *this < r ); }
  };

// Needed for ArrayRef::operator& since a bug in CW Pro 5 prevents defining
// ArrayRef::operator& outside the (specialized) class definition.
template < class Element >
inline ArrayPtr< Element > MakeArrayPtr( Element *begin, Element *end )
  {
	return ArrayPtr<Element>( begin, end );
  }

#endif
