// LinkedListBase.cp

#ifndef LinkedListBase_h
#include "LinkedListBase.h"
#endif

LinkedListBase< true >::Link LinkedListBase< true >::Link::singular;

LinkedListBase< false >::LinkedListBase( LinkedListBase&, iterator what )
  {
	Link& link = *what;
	link.Unlink();
	Twist( link, endLink );
  }

LinkedListBase< false >::LinkedListBase( LinkedListBase&,
													  iterator b,
													  iterator e )
  {
	if ( b == e )
		return;
	
	Link& link = *b;
	Twist( link, *e );
	Twist( link, endLink );
  }

LinkedListBase< false >::size_type
LinkedListBase< false >::size() const
  {
	size_type n = 0;
	for ( const_iterator i = begin(); i != end(); i++ )
		n++;
	return n;
  }

LinkedListBase< false >::iterator
LinkedListBase< false >::insert( iterator where, Link& what )
  {
	what.Unlink();
	Twist( what, *where );
	return where;
  }

void
LinkedListBase< false >::splice( iterator where,
											iterator b,
											iterator e )
  {
	if ( where == b || b == e )
		return;
	
	Link& link = *b;
	Twist( link, *e );
	Twist( link, *where );
  }

LinkedListBase< false >::iterator
LinkedListBase< false >::erase( iterator b, iterator e )
  {
	while ( b != e )
		erase( b++ );
	return e;
  }

void
LinkedListBase< false >::reverse()
  {
	LinkedListBase source = *this;
	while ( !source.empty() )
		splice( begin(), source.begin() );
  }


LinkedListBase< true >::Link::Link()
  : end( this )
  {
  }

LinkedListBase< true >::Link::Link( Link& source )
  : end( source.end )
  {
	source.end = end;
	
	if ( end == &source )
		end = this;
  }

LinkedListBase< true >::Link::Link( ResourceTransfer<Link> source )
  : end( source->end )
  {
	source->end = end;
	
	if ( end == &*source )
		end = this;
  }

LinkedListBase< true >::Link&
LinkedListBase< true >::Link::operator=( Link& source )
  {
	end = source.end;
	source.end = &source;
	
	if ( end == &source )
		end = this;
	
	RingLink::operator=( source );
	return *this;
  }

void
LinkedListBase< true >::Link::swap( Link& source )
  {
	std::swap( end, source.end );
	
	if ( end == &source )
		end = this;
	
	if ( source.end == this )
		source.end = &source;

	RingLink::swap( source );
  }

void
LinkedListBase< true >::Link::Unlink()
  {
	RingLink::Unlink();
	end = this;
  }

bool
LinkedListBase< true >::IsValidRange( const_iterator b, const_iterator e )
  {
	Assert( b.ComparableTo( e ) );
	
	for ( ; b != e; ++b )
		if ( !b.HasStar() )
			return false;
	
	return true;
  }

void
LinkedListBase< true >::FixEnds()
  {
	for ( iterator b = begin(); b != end(); b++ )
		b->end = &endLink;
  }

LinkedListBase< true >::LinkedListBase( LinkedListBase& s )
  : endLink( s.endLink )
  {
	FixEnds();
  }

LinkedListBase< true >::LinkedListBase( ResourceTransfer<LinkedListBase> s )
  : endLink( s->endLink )
  {
	FixEnds();
  }

LinkedListBase< true >::LinkedListBase( LinkedListBase& source, iterator what )
  {
	Assert( source.Contains( what ) );
	Assert( what != source.end() );
	
	Link& link = *what;
	Twist( link, *++what );
	Twist( link, endLink );
	
	link.end = &endLink;
  }

LinkedListBase< true >::LinkedListBase( LinkedListBase& source,
													 iterator b,
													 iterator e )
  {
	Assert( source.Contains( b ) );
	Assert( source.Contains( e ) );
	Assert( b.ComparableTo( e ) );
	Assert( IsValidRange( b, e ) );
	
	if ( b == e )
		return;
	
	Link& link = *b;
	Twist( link, *e );
	Twist( link, endLink );
	
	for ( ; b != end(); ++b )
		b->end = &endLink;
  }

LinkedListBase< true >&
LinkedListBase< true >::operator=( LinkedListBase& s )
  {
	endLink = s.endLink;
	FixEnds();
	return *this;
  }

void
LinkedListBase< true >::swap( LinkedListBase& source )
  {
	endLink.swap( source.endLink );
	FixEnds();
	source.FixEnds();
  }

LinkedListBase< true >::size_type
LinkedListBase< true >::size() const
  {
	size_type n = 0;
	for ( const_iterator i = begin(); i != end(); i++ )
		n++;
	return n;
  }
			
LinkedListBase< true >::iterator
LinkedListBase< true >::insert( iterator where, Link& what )
  {
	Assert( Contains( where ) );
	Assert( !Contains( what ) );
	Assert( what.end == &what );
	
	Twist( what, *where );
	what.end = &endLink;
	
	return where;
  }

void
LinkedListBase< true >::splice( iterator where,
										  iterator b,
										  iterator e )
  {
	Assert( Contains( where ) );
	Assert( b.ComparableTo( e ) );
	Assert( IsValidRange( b, e ) );
	
	if ( where == b || b == e )
		return;
	
	Link& link = *b;
	Twist( link, *e );
	Twist( link, *where );
	
	for ( ; b != where; ++b )
		b->end = &endLink;
  }

void
LinkedListBase< true >::splice( iterator where,
										  LinkedListBase& source,
										  iterator what )
  {
	Assert( source.Contains( what ) );
	splice( where, what );
  }

void
LinkedListBase< true >::splice( iterator where,
										  LinkedListBase& source,
										  iterator b,
										  iterator e )
  {
	Assert( source.Contains( b ) );
	Assert( source.Contains( e ) );
	splice( where, b, e );
  }

LinkedListBase< true >::iterator
LinkedListBase< true >::erase( iterator where )
  {
	Assert( Contains( where ) );
	Assert( where != end() );

	(where++)->Unlink();
	return where;
  }

LinkedListBase< true >::iterator
LinkedListBase< true >::erase( iterator b, iterator e )
  {
	Assert( Contains( b ) );
	Assert( Contains( e ) );

	while ( b != e )
		erase( b++ );
	
	return e;
  }

void
LinkedListBase< true >::reverse()
  {
	LinkedListBase source = *this;
	while ( !source.empty() )
		splice( begin(), source.begin() );
  }
