// ObservableConstProperty.h

#ifndef ObservableConstProperty_h
#define ObservableConstProperty_h

#ifndef Observable_h
#include "Observable.h"
#endif
#ifndef ConstProperty_h
#include "ConstProperty.h"
#endif

template < class Value >
class ObservableConstProperty: public Observable< void(*)( Value ) >
										 
  {
	public:
		typedef ::Observable< void(*)( Value ) > Observable;
		typedef ::ConstProperty< Value > ConstProperty;
	
		typedef ConstProperty::GetterClosure GetterClosure;

	private:
		ConstProperty property;
		
	public:
		ObservableConstProperty()
		  {}

		ObservableConstProperty( GetterClosure closure )
		  : property( closure )
		  {}
		
		template < class FR >
		ObservableConstProperty( FR (*function)() )
		  : property( function )
		  {}
		
		template < class Obj, class FR, class FObj >
		ObservableConstProperty( const Obj& object, FR (*function)( const FObj& ) )
		  : property( object, function )
		  {}
		
		template < class Obj, class MR, class MObj >
		ObservableConstProperty( const Obj& object, MR (MObj::*method)() const )
		  : property( object, method )
		  {}

		template < class ActualValue >
		ObservableConstProperty( const ActualValue& object )
		  : property( object )
		  {}
		
		operator ConstProperty() const				{ return property; }
		GetterClosure Getter() const					{ return property.Getter(); }
		
		Value Get() const									{ return property.Get(); }
		operator Value() const							{ return property.Get(); }

		void BroadcastValue() const					{ Observable::Broadcast( Get() ); }
  };

#endif
