// ObservableProperty.h

#ifndef ObservableProperty_h
#define ObservableProperty_h

#ifndef ObservableConstProperty_h
#include "ObservableConstProperty.h"
#endif
#ifndef Property_h
#include "Property.h"
#endif

template < class Value >
class ObservableProperty: public ObservableConstProperty< Value >
  {
	public:
		typedef ::Observable< void(*)( Value ) > Observable;
		typedef ::ObservableConstProperty< Value > ObservableConstProperty;
		typedef ::ConstProperty< Value > ConstProperty;
		typedef ::PropertySetter< Value > PropertySetter;
		typedef ::Property< Value > Property;
	
		typedef ConstProperty::GetterClosure GetterClosure;
		typedef PropertySetter::SetterClosure SetterClosure;
		
	private:		
		PropertySetter setter;
		
	public:
		ObservableProperty( ConstProperty getter, PropertySetter setter )
		  : ObservableConstProperty( getter ),
			 setter( setter )
		  {}
		
		ObservableProperty( GetterClosure getter, SetterClosure setter )
		  : ObservableConstProperty( getter ),
			 setter( setter )
		  {}
		
		template < class V >
		ObservableProperty( V (*getter)(), void (*setter)(V) )
		  : ObservableConstProperty( getter ),
			 setter( setter )
		  {}
		
		template < class V >
		ObservableProperty( V (*getter)(), void (*setter)( const V& ) )
		  : ObservableConstProperty( getter ),
			 setter( setter )
		  {}
		
		template < class Obj, class V, class FObj >
		ObservableProperty( Obj& object, V (*getter)( const FObj& ), void (*setter)( FObj&, V ) )
		  : ObservableConstProperty( object, getter ),
			 setter( object, setter )
		  {}
		
		template < class Obj, class V, class FObj >
		ObservableProperty( Obj& object, V (*getter)( const FObj& ), void (*setter)( FObj&, const V& ) )
		  : ObservableConstProperty( object, getter ),
			 setter( object, setter )
		  {}
		
		template < class Obj, class V, class MObj >
		ObservableProperty( Obj& object, V (MObj::*getter)() const, void (MObj::*setter)( V ) )
		  : ObservableConstProperty( object, getter ),
			 setter( object, setter )
		  {}
		
		template < class Obj, class V, class MObj >
		ObservableProperty( Obj& object, V (MObj::*getter)() const, void (MObj::*setter)( const V& ) )
		  : ObservableConstProperty( object, getter ),
			 setter( object, setter )
		  {}

		template < class ActualValue >
		ObservableProperty( ActualValue& object )
		  : ObservableConstProperty( object ),
			 setter( object )
		  {}
		
		SetterClosure Setter() const									{ return SetterClosure( *this, &ObservableProperty::Set ); }
		
		operator PropertySetter() const																{ return PropertySetter( Setter() ); }
		operator Property() const																		{ return Property( Getter(), Setter() ); }

		void Set( Value v ) const																		{ setter.Set( v ); BroadcastValue(); }
		
		const ObservableProperty& operator=( Value v ) const									{ Set( v ); return *this; }
		const ObservableProperty& operator=( const ObservableProperty& v ) const		{ Set( v ); return *this; }

		template < class R > const Property& operator+=( R r ) const						{ return *this = Get() + r; }
		template < class R > const Property& operator-=( R r ) const						{ return *this = Get() - r; }
		
		template < class R > const Property& operator*=( R r ) const						{ return *this = Get() * r; }
		template < class R > const Property& operator/=( R r ) const						{ return *this = Get() / r; }
		template < class R > const Property& operator%=( R r ) const						{ return *this = Get() % r; }
		
		template < class R > const Property& operator&=( R r ) const						{ return *this = Get() & r; }
		template < class R > const Property& operator|=( R r ) const						{ return *this = Get() | r; }
		template < class R > const Property& operator^=( R r ) const						{ return *this = Get() ^ r; }
		
		template < class R > const Property& operator>>=( R r ) const						{ return *this = Get() >> r; }
		template < class R > const Property& operator<<=( R r ) const						{ return *this = Get() << r; }
  };

#endif
