// ConstProperty.h

#ifndef ConstProperty_h
#define ConstProperty_h

#ifndef Closure_h
#include "Closure.h"
#endif

template < class Value > class ConstProperty;
template < class Value > class ConstPropertyPtr;
template < class Value > class Property;

template < class Value >
class ConstProperty
  {
	friend class ConstPropertyPtr< Value >;
	friend class Property< Value >;

	public:
		typedef Closure< Value(*)() > GetterClosure;
	
	private:
		GetterClosure get;
		
		// not implemented:
			ConstProperty& operator=( const ConstProperty& );
	
		void Reseat( const ConstProperty& source )
		  {
			get = source.get;
		  }
		
		template < class ActualValue >
		static ActualValue SimplyGet( const ActualValue& object )
		  {
			return object;
		  }
		
		template < class ActualValue >
		static GetterClosure MakeSimpleGet( const ActualValue& object )
		  {
			// This routine could be more simply inlined into the constructor
			// where it's used, but keeping it separate helps CW 5.3 avoid confusion.
			typedef ActualValue (*SimpleGetter)( const ActualValue& );
			SimpleGetter getter = &SimplyGet<ActualValue>;
			return GetterClosure( object, getter );
		  }
		
	public:
		ConstProperty()
		  {}
		
		ConstProperty( GetterClosure closure )
		  : get( closure )
		  {}
		
		template < class FR >
		ConstProperty( FR (*function)() )
		  : get( function )
		  {}
		
		template < class Obj, class FR, class FObj >
		ConstProperty( const Obj& object, FR (*function)( const FObj& ) )
		  : get( object, function )
		  {}
		
		template < class Obj, class MR, class MObj >
		ConstProperty( const Obj& object, MR (MObj::*method)() const )
		  : get( object, method )
		  {}

		template < class ActualValue >
		ConstProperty( const ActualValue& object )
		  : get( MakeSimpleGet( object ) )
		  {}
		
		GetterClosure Getter() const				{ return get; }
		
		Value Get() const								{ return get(); }
		operator Value() const						{ return get(); }
  };

#endif
