// Property.h

#ifndef Property_h
#define Property_h

#ifndef ConstProperty_h
#include "ConstProperty.h"
#endif
#ifndef PropertySetter_h
#include "PropertySetter.h"
#endif

template < class Value > class Property;
template < class Value > class PropertyPtr;

template < class Value >
class Property: public ConstProperty< Value >,
					 public PropertySetter< Value >
  {
	friend class PropertyPtr< Value >;

	public:
		typedef ::ConstProperty< Value > ConstProperty;
		typedef ::PropertySetter< Value > PropertySetter;
	
		typedef ConstProperty::GetterClosure GetterClosure;
		typedef PropertySetter::SetterClosure SetterClosure;
	
	private:
		void Reseat( const Property& source )
		  {
			ConstProperty::Reseat( source );
			PropertySetter::Reseat( source );
		  }
		
	public:
		Property()
		  {}

		Property( ConstProperty getter, PropertySetter setter )
		  : ConstProperty( getter ),
			 PropertySetter( setter )
		  {}
		
		Property( GetterClosure getter, SetterClosure setter )
		  : ConstProperty( getter ),
			 PropertySetter( setter )
		  {}
		
		template < class V >
		Property( V (*getter)(), void (*setter)(V) )
		  : ConstProperty( getter ),
			 PropertySetter( setter )
		  {}
		
		template < class V >
		Property( V (*getter)(), void (*setter)( const V& ) )
		  : ConstProperty( getter ),
			 PropertySetter( setter )
		  {}
		
		template < class Obj, class V, class FObj >
		Property( Obj& object, V (*getter)( const FObj& ), void (*setter)( FObj&, V ) )
		  : ConstProperty( object, getter ),
			 PropertySetter( object, setter )
		  {}
		
		template < class Obj, class V, class FObj >
		Property( Obj& object, V (*getter)( const FObj& ), void (*setter)( FObj&, const V& ) )
		  : ConstProperty( object, getter ),
			 PropertySetter( object, setter )
		  {}
		
		template < class Obj, class V, class MObj >
		Property( Obj& object, V (MObj::*getter)() const, void (MObj::*setter)( V ) )
		  : ConstProperty( object, getter ),
			 PropertySetter( object, setter )
		  {}
		
		template < class Obj, class V, class MObj >
		Property( Obj& object, V (MObj::*getter)() const, void (MObj::*setter)( const V& ) )
		  : ConstProperty( object, getter ),
			 PropertySetter( object, setter )
		  {}

		template < class ActualValue >
		Property( ActualValue& object )
		  : ConstProperty( object ),
			 PropertySetter( object )
		  {}
		
		const Property& operator=( Value v ) const							{ Set( v ); return *this; }
		const Property& operator=( const Property& v ) const				{ Set( v ); return *this; }
		
		template < class R > const Property& operator+=( R r ) const	{ return *this = Get() + r; }
		template < class R > const Property& operator-=( R r ) const	{ return *this = Get() - r; }
		
		template < class R > const Property& operator*=( R r ) const	{ return *this = Get() * r; }
		template < class R > const Property& operator/=( R r ) const	{ return *this = Get() / r; }
		template < class R > const Property& operator%=( R r ) const	{ return *this = Get() % r; }
		
		template < class R > const Property& operator&=( R r ) const	{ return *this = Get() & r; }
		template < class R > const Property& operator|=( R r ) const	{ return *this = Get() | r; }
		template < class R > const Property& operator^=( R r ) const	{ return *this = Get() ^ r; }
		
		template < class R > const Property& operator>>=( R r ) const	{ return *this = Get() >> r; }
		template < class R > const Property& operator<<=( R r ) const	{ return *this = Get() << r; }
  };

#endif
