// PropertySetter.h

#ifndef PropertySetter_h
#define PropertySetter_h

#ifndef Closure_h
#include "Closure.h"
#endif

template < class Value > class PropertySetter;
template < class Value > class PropertySetterPtr;
template < class Value > class Property;

template < class Value >
class PropertySetter
  {
	friend class PropertySetterPtr< Value >;
	friend class Property< Value >;

	public:
		typedef Closure< void(*)(Value) > SetterClosure;
	
	private:
		SetterClosure set;
		
		// not implemented:
			PropertySetter& operator=( const PropertySetter& );
	
		void Reseat( const PropertySetter& source )
		  {
			set = source.set;
		  }
		
		template < class ActualValue >
		static void SimplySet( ActualValue& object, ActualValue v )
		  {
			object = v;
		  }
		
		template < class ActualValue >
		static SetterClosure MakeSimpleSet( ActualValue& object )
		  {
			typedef void (*SimpleSetter)( ActualValue&, ActualValue );
			SimpleSetter setter = &SimplySet<ActualValue>;
			return SetterClosure( object, setter );
		  }
		
	public:
		PropertySetter()
		  {}
		
		PropertySetter( SetterClosure setter )
		  : set( setter )
		  {}
		
		template < class V >
		PropertySetter( void (*setter)(V) )
		  : set( setter )
		  {}
		
		template < class V >
		PropertySetter( void (*setter)( const V& ) )
		  : set( setter )
		  {}
		
		template < class Obj, class V, class FObj >
		PropertySetter( Obj& object, void (*setter)( FObj&, V ) )
		  : set( object, setter )
		  {}
		
		template < class Obj, class V, class FObj >
		PropertySetter( Obj& object, void (*setter)( FObj&, const V& ) )
		  : set( object, setter )
		  {}
		
		template < class Obj, class V, class MObj >
		PropertySetter( Obj& object, void (MObj::*setter)( V ) )
		  : set( object, setter )
		  {}
		
		template < class Obj, class V, class MObj >
		PropertySetter( Obj& object, void (MObj::*setter)( const V& ) )
		  : set( object, setter )
		  {}

		template < class ActualValue >
		PropertySetter( ActualValue& object )
		  : set( MakeSimpleSet( object ) )
		  {}

		SetterClosure Setter() const								{ return set; }
		
		void Set( Value v ) const									{ set( v ); }
		const PropertySetter& operator=( Value v ) const	{ set( v ); return *this; }
  };

#endif
