// AutoArrayPtr.h

#ifndef AutoArrayPtr_h
#define AutoArrayPtr_h

#ifndef ArrayPtr_h
#include "ArrayPtr.h"
#endif
#ifndef ResourceTransfer_h
#include "ResourceTransfer.h"
#endif

template < class Element >
class AutoArrayPtr
  {
	public:
		typedef ArrayPtr< Element > 							ArrayPointer;

		typedef ArrayPointer::ArrayReference 				ArrayReference;
		typedef ArrayPointer::value_type						value_type;
		typedef ArrayPointer::reference						reference;
		typedef ArrayPointer::const_reference				const_reference;
		typedef ArrayPointer::difference_type				difference_type;
		typedef ArrayPointer::size_type						size_type;
		typedef ArrayPointer::const_iterator				const_iterator;
		typedef ArrayPointer::iterator						iterator;
		typedef ArrayPointer::reverse_iterator				reverse_iterator;
		typedef ArrayPointer::const_reverse_iterator		const_reverse_iterator;
		
	private:
		ArrayPointer array;
		
	public:
		AutoArrayPtr()																{}
		AutoArrayPtr( value_type *begin, value_type *end )				: array( begin, end )		{}
		AutoArrayPtr( AutoArrayPtr& s )										: array( s.release() )		{}
		AutoArrayPtr( ResourceTransfer<AutoArrayPtr> s )				: array( s->release() )		{}
		
		~AutoArrayPtr()															{ delete[] array; }
		
		operator ResourceTransfer<AutoArrayPtr>()							{ return *this; }
		
		AutoArrayPtr& operator=( AutoArrayPtr& s )						{ reset( s.release() ); return *this; }
		AutoArrayPtr& operator=( ResourceTransfer<AutoArrayPtr> s )	{ return operator=( *s ); }
		
		ArrayPointer get() const												{ return array; }
		ArrayPointer release()													{ ArrayPtr a( array ); array = ArrayPointer(); return a; }
		
		void reset( const ArrayPointer& s )									{ delete[] array; array = s; }
		void reset()																{ reset( ArrayPointer() ); }

		const ArrayReference& operator*() const							{ return *array; }
		const ArrayReference *operator->() const							{ return array.operator->(); }
		reference operator[]( size_type n ) const							{ return array[n]; }
  };

template < class Element >
AutoArrayPtr<Element> CreateArray( typename AutoArrayPtr<Element>::size_type length )
  {
	Element *begin = new Element[ length ];
	return AutoArrayPtr<Element>( begin, begin + length );
  }

#endif
