// SharingArrayPtr.h

#ifndef SharingArrayPtr_h
#define SharingArrayPtr_h

#ifndef AutoArrayPtr_h
#include "AutoArrayPtr.h"
#endif
#ifndef RingLink_h
#include "RingLink.h"
#endif

template < class Element >
class SharingArrayPtr
  {
	public:
		typedef ArrayPtr< Element > 							ArrayPointer;
		typedef AutoArrayPtr< Element > 						AutoArrayPointer;

		typedef ArrayPointer::ArrayReference 				ArrayReference;
		typedef ArrayPointer::value_type						value_type;
		typedef ArrayPointer::reference						reference;
		typedef ArrayPointer::const_reference				const_reference;
		typedef ArrayPointer::difference_type				difference_type;
		typedef ArrayPointer::size_type						size_type;
		typedef ArrayPointer::const_iterator				const_iterator;
		typedef ArrayPointer::iterator						iterator;
		typedef ArrayPointer::reverse_iterator				reverse_iterator;
		typedef ArrayPointer::const_reverse_iterator		const_reverse_iterator;
		
	private:
		ArrayPointer array;
		mutable RingLink link;
		
	public:
		SharingArrayPtr()											{}
		SharingArrayPtr( const SharingArrayPtr& s )
		  : array( s.array )
		  {
			Twist( link, s.link );
		  }
				
		SharingArrayPtr& operator=( const SharingArrayPtr& s )
		  {
		 	reset();
		 	array = s.array;
		 	Twist( link, s.link );
		 	return *this;
		  }

		void reset()
		  {
			if ( link.Alone() )
				delete[] array;
			 else
				link.Unlink();
			array = ArrayPointer();
		  }
		
		~SharingArrayPtr()										{ reset(); }
				
		operator value_type*() const							{ return array; }

		const ArrayReference& operator*() const			{ return *array; }
		const ArrayReference *operator->() const			{ return array.operator->(); }
		reference operator[]( size_type n ) const			{ return array[n]; }
  };

#endif
