// SharingPtr.h

#ifndef SharingPtr_h
#define SharingPtr_h

#include <memory>
#include <algorithm>
#ifndef RingLink_h
#include "RingLink.h"
#endif

template < class T >
class SharingPtr
  {
	private:
		T *target;
		mutable RingLink link;
			
	public:
		SharingPtr()													: target( 0 )						{}

		SharingPtr( const SharingPtr& b )						: target( b.target )				{ Twist( link, b.link ); }

		template < class T2 >
		SharingPtr( const SharingPtr<T2>& b )					: target( b.target )				{ Twist( link, b.link ); }

		template < class T2 >
		SharingPtr( std::auto_ptr<T2> b )						: target( b.release() ) 		{ Twist( link, b.link ); }
		
		~SharingPtr()													{ reset(); }
				
		SharingPtr& operator=( const SharingPtr& b )			{ reset(); target = b.target; Twist( link, b.link ); return *this; }

		template < class T2 >
		SharingPtr& operator=( const SharingPtr<T2>& b )	{ reset(); target = b.target; Twist( link, b.link ); return *this; }
		
		template < class T2 >
		SharingPtr& operator=( std::auto_ptr<T2> b )			{ reset(); target = b.target; Twist( link, b.link ); return *this; }

		T& operator*() const											{ return *target; }
		T *operator->() const										{ return target; }
		operator T*() const											{ return target; }
		
		void swap( SharingPtr<T>& b )								{ std::swap( target, b.target );	link.swap( b.link ); }
		
		void reset()
		  {
			if ( link.Alone() )
				delete target;
			 else
				link.Unlink();
			target = 0;
		  }
  };

namespace std
  {
	template < class T >
	inline void swap( SharingPtr<T>& a, SharingPtr<T>& b )
	  {
		a.swap( b );
	  }
  }

#endif
